package com.perforce.workshop.tjuricek.p4oauth.test_server;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.perforce.workshop.tjuricek.p4java_ext.FileSummary;
import com.perforce.workshop.tjuricek.p4java_ext.P4Connection;
import com.perforce.workshop.tjuricek.p4java_ext.P4Files;
import org.eclipse.jetty.server.Request;
import org.eclipse.jetty.server.handler.AbstractHandler;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.List;
import java.util.stream.Collectors;

import static org.apache.commons.io.IOUtils.closeQuietly;

/**
 * This is kind of a stupid helper for a single command but there you go.
 * <p>
 * We take a POST request to /list that includes our p4d token and login
 * and performs a "files" command just to see if we have the right auth
 * token.
 */
public class ListerHelper extends AbstractHandler {

    private ObjectMapper objectMapper = new ObjectMapper();

    @Override
    public void handle(String target, Request baseRequest,
                       HttpServletRequest request, HttpServletResponse response)
            throws IOException, ServletException {

        if (baseRequest.getMethod().equals("POST") &&
                baseRequest.getPathInfo().equals("/list")) {

            baseRequest.setHandled(true);

            String clientId = request.getParameter("client_id");
            String token = request.getParameter("token");

            P4Connection conn = P4Connection.create("127.0.0.1", 1666, false,
                    "test server", "1.0");

            try {
                conn.connectAs(clientId);
                conn.getOptionsServer().setAuthTicket(token);

                P4Files p4Files = P4Files.create(conn);

                List<FileSummary> summaryList = p4Files.list("//depot/...");

                List<String> fileNames =
                        summaryList.stream()
                                .map(fs -> fs.getDepotFile())
                                .collect(Collectors.toList());

                response.setContentType("application/json");
                objectMapper.writeValue(response.getOutputStream(), fileNames);

            } catch (Exception e) {
                e.printStackTrace();
                response.sendError(500, e.getMessage());
            } finally {
                closeQuietly(conn);
            }
        }
    }
}
