package com.perforce.workshop.tjuricek.p4oauth.storage;

import com.perforce.workshop.tjuricek.p4oauth.config.P4OAuthConfig;
import com.perforce.workshop.tjuricek.p4oauth.models.AuthorizationCode;

import java.io.File;
import java.io.IOException;
import java.util.Optional;

/**
 * Implements reading and writing various AuthorizationCodes.
 * <p>
 * Currently backed by the filesystem.
 */
public class AuthorizationCodeStore implements FilesystemStore {

    private P4OAuthConfig config;

    public AuthorizationCodeStore(P4OAuthConfig config) {
        this.config = config;
    }

    public Optional<AuthorizationCode> read(String code) throws IOException {
        File codePath = getCodeFile(code);
        if (codePath.exists()) {
            return Optional.of(readYamlFile(AuthorizationCode.class, codePath));
        } else {
            return Optional.empty();
        }
    }

    public void delete(String code) throws IOException {
        File codeFile = getCodeFile(code);
        if (codeFile.exists()) {
            codeFile.delete();
        }
    }

    public void update(AuthorizationCode authCode) throws IOException {
        File codePath = getCodeFile(authCode.getAuthorizationCode());
        ensureParentExists(codePath);
        writeYamlFile(authCode, codePath);
    }

    public File getCodeFile(String code) {
        return getFile("authorizations", code);
    }

    @Override
    public P4OAuthConfig getConfig() {
        return config;
    }
}
