package com.perforce.workshop.tjuricek.p4oauth.util;

import java.util.function.Supplier;

/**
 * Wraps a Supplier with thread local caching.
 */
public class ThreadMemoizingSupplier<T> implements Supplier<T> {

    private Supplier<T> wrapped;
    private T cached = null;
    private ThreadLocal<T> threadLocal = new ThreadLocal<T>() {
        @Override
        protected T initialValue() {
            cached = wrapped.get();
            return cached;
        }
    };


    public ThreadMemoizingSupplier(Supplier<T> wrapped) {
        this.wrapped = wrapped;
    }

    /**
     * Returns the cached type. If it's null, then the type was not used in this
     * thread.
     */
    public T getCached() {
        return cached;
    }

    public T release() {
        T tmp = cached;
        cached = null;
        return tmp;
    }

    /**
     * Will always fire the wrapped supplier and return the value, which is
     * then cached on the thread.
     */
    @Override
    public T get() {
        return threadLocal.get();
    }

    /**
     * Returns the underlying supplier used to fetch the value.
     */
    public Supplier<T> getWrapped() {
        return wrapped;
    }
}
